@@ -6,11 +6,11 @@ from codingnow.game.platform.player import *
 
 class Bullet(pygame.sprite.Sprite):
 	
-    def __init__(self,screen:Surface,filename,player:Player):
+    def __init__(self,screen:Surface,img,player:Player):
         pygame.sprite.Sprite.__init__(self)
         self.screen = screen
         self.direction = player.direction*2
-        img = pygame.image.load(f'{filename}').convert_alpha()
+        # img = pygame.image.load(f'{filename}').convert_alpha()
         self.image_src = pygame.transform.scale(img,(40,30))
         self.image = self.image_src
         self.rect = self.image.get_rect()
