@@ -6,33 +6,49 @@ import random
 
 class Player():
     speed = 0
+    SPEED_DEF = 2
+    JUMP_DEF = 15
     JUMP = 15
     jumped = False
     jump_y = 0
     score = 0
     level = 1
+    level_real = 1
     gameover = False
     direction = 2
+    hp = 3
     weapons = []
     imgs = {}
+    jump_cnt = 0
+    gametime = 0
+    gametime_tick = 0
+    MissionCompleted = False
+    is_start = True
+    jump_set_level = {}
+    speed_set_level = {}
+    
     start_position = {1:{'x' : 20,'y':510}}
-    def __init__(self,parent,screen:Surface,filename:str, width:int, height:int,flip: bool) -> None:
+    def __init__(self,parent,screen:Surface,filename:str=None, width:int=0, height:int=0,flip: bool=False) -> None:
         self.parent = parent
         self.screen = screen
         self.img_gameover = None  
         self.rect = None
         self.image = None
+        self.JUMP = self.JUMP_DEF
+        self.speed = self.SPEED_DEF
+        # if filename is not None:
         self.image = self.set_img(self.level, filename,flip,width,height)
         self.img_idex = 0
         
-        # self.image = self.get_img()
-        # self.rect = self.image.get_rect()
+        # if filename is not None:
         self.game_reset(True)
-        self.weapon_pressed = False
-        # self.image_bullet = None        
+        self.weapon_pressed = False      
         self.msg_level_text = None
         self.msg_score_text = None
         self.msg_weapon_text = None
+        self.msg_hp_text = None
+        self.msg_jumpcnt_text = None
+        self.msg_gametime_text = None
         self.mfont20 = pygame.font.SysFont('malgungothic', 20)
         self.mfont30 = pygame.font.SysFont('malgungothic', 30)
         self.mfont40 = pygame.font.SysFont('malgungothic', 40)
@@ -74,7 +90,61 @@ class Player():
         rect.y = self.rect.y
         self.rect = rect
         return img
+    
+    def set_jump(self, level:int, jump:int):
+        self.jump_set_level[level] = jump
         
+    def set_speed(self, level:int, speed:int):
+        self.speed_set_level[level] = speed
+        
+    def draw_persion(self, width:int=60, height:int=60): 
+        temp_surface = pygame.Surface((width, height), pygame.SRCALPHA)
+
+        # 기준 비율
+        head_ratio = 0.15   # 머리 반지름 비율 (세로 기준)
+        body_ratio = 0.35   # 몸통 길이 비율
+        arm_offset_ratio = 0.25  # 팔 길이 비율
+        leg_ratio = 0.4     # 다리 길이 비율
+
+        # 중심 좌표 계산
+        center_x = width // 2
+        top_y = int(height * 0.05)  # 머리 상단 위치
+
+        # 비율에 따른 실제 크기
+        head_radius = int(height * head_ratio)
+        body_length = int(height * body_ratio)
+        arm_length = int(height * arm_offset_ratio)
+        leg_length = int(height * leg_ratio)
+
+        # 색상
+        BLACK = (0, 0, 0)
+        SKIN = (255, 224, 189)
+
+        # 머리 중심 위치
+        head_center_y = top_y + head_radius
+        pygame.draw.circle(temp_surface, SKIN, (center_x, head_center_y), head_radius)
+
+        # 몸통
+        body_start = (center_x, head_center_y + head_radius)
+        body_end = (center_x, body_start[1] + body_length)
+        pygame.draw.line(temp_surface, BLACK, body_start, body_end, 2)
+
+        # 팔
+        arm_start = (center_x, body_start[1] + int(body_length * 0.2))
+        pygame.draw.line(temp_surface, BLACK, arm_start,
+                        (center_x - arm_length, arm_start[1] + arm_length), 2)
+        pygame.draw.line(temp_surface, BLACK, arm_start,
+                        (center_x + arm_length, arm_start[1] + arm_length), 2)
+
+        # 다리
+        leg_start = body_end
+        pygame.draw.line(temp_surface, BLACK, leg_start,
+                        (center_x - leg_length, leg_start[1] + leg_length), 2)
+        pygame.draw.line(temp_surface, BLACK, leg_start,
+                        (center_x + leg_length, leg_start[1] + leg_length), 2)
+
+        return temp_surface
+            
     def set_img(self, level:int, filename:str, flip:bool=False, width:int=60, height:int=60):  
         
         self.imgs[level] = {
@@ -82,7 +152,12 @@ class Player():
             'right':[],
             }
          
-        img = pygame.image.load(f'{filename}').convert_alpha()     
+        filename = self.parent.get_folder_img(filename)
+        # print(f'Player image : {filename}')
+        if filename is None:
+            img = self.draw_persion(width, height)
+        else:          
+            img = pygame.image.load(f'{filename}').convert_alpha()     
         if flip:
             image_src_l = pygame.transform.scale(img,(width,height))
             image_src_r = pygame.transform.flip(image_src_l,True,False)
@@ -104,19 +179,35 @@ class Player():
         return image_src_r
         
     def set_gameover_image(self,filename):
+        filename = self.parent.get_folder_img(filename)
         img = pygame.image.load(f'{filename}').convert_alpha()
         self.img_gameover = pygame.transform.scale(img,(self.rect.width,self.rect.height))
         
     def game_reset(self, reload_map):
         self.score = 0
-        self.level = 1
-        
+        self.level = 1      
+        self.level_real = 1  
+        self.hp = 3
+        self.weapons.clear()
+        self.jump_cnt = 0
         self.image = self.get_img(self.level)
-        
+        self.MissionCompleted = False
+        self.gametime = 0
+        self.gametime_tick = pygame.time.get_ticks() + 10
         self.rect = self.image.get_rect()
         self.rect.x,self.rect.y = self.get_position(self.level) 
         self.rect_pre = self.rect.copy()
         
+        if self.level in self.jump_set_level:
+            self.JUMP = self.jump_set_level[self.level]
+        else:
+            self.JUMP = self.JUMP_DEF
+        
+        if self.level in self.speed_set_level:
+            self.speed = self.speed_set_level[self.level]
+        else:
+            self.speed = self.SPEED_DEF
+            
         self.gameover = False
         if reload_map:
             self.parent.map_change(self.level)
@@ -126,18 +217,23 @@ class Player():
     #     self.image_bullet = pygame.transform.scale(img,(40,30))
         
     def set_snd_weapon(self,filename):
+        filename = self.parent.get_folder_snd(filename)
         self.snd_dic['weapon'] = pygame.mixer.Sound(filename)
         
     def set_snd_coin(self,filename):
+        filename = self.parent.get_folder_snd(filename)
         self.snd_dic['coin'] = pygame.mixer.Sound(filename)
         
     def set_snd_jump(self,filename):
+        filename = self.parent.get_folder_snd(filename)
         self.snd_dic['jump'] = pygame.mixer.Sound(filename)
         
     def set_snd_game_over(self,filename):
+        filename = self.parent.get_folder_snd(filename)
         self.snd_dic['game_over'] = pygame.mixer.Sound(filename)
         
     def set_snd_monster(self,filename):
+        filename = self.parent.get_folder_snd(filename)
         self.snd_dic['monster'] = pygame.mixer.Sound(filename)
     
     def rotate(self, angle):
@@ -207,6 +303,7 @@ class Player():
             
     def jump(self):
         if self.jumped == False:
+            self.jump_cnt += 1
             if self.snd_dic['jump'] is not None:
                 self.snd_dic['jump'].play()
             self.jump_y = self.JUMP * (-1)
@@ -235,11 +332,18 @@ class Player():
             brect = block.rect.copy()
             if brect.colliderect(xc):
                 temp_yc = xc.copy()
-                temp_yc.y -= temp_yc.height
+                temp_yc.y -= brect.height
                 if not brect.colliderect(temp_yc):
                     top = brect.top
                 else:
                     return 0
+            else:
+                temp_yc = xc.copy()
+                temp_yc.y -= brect.height
+                # pygame.draw.rect(self.screen,(0,0,0),temp_yc)
+                if brect.colliderect(temp_yc):
+                    return 0
+                
         return top
                 
             
@@ -258,15 +362,15 @@ class Player():
             is_up = False
             is_down = False
                 
-            if brect.bottom < self.rect_pre.top: #블럭 아래 있음
+            if brect.bottom <= self.rect_pre.top: #블럭 아래 있음
                 brect.top -= self.JUMP
-                brect.height += self.JUMP+5
+                brect.height += self.JUMP
                 is_down = True
             else:#블럭 위에 있음
                 if self.jumped:
                     brect.height += self.JUMP
                 is_up = True
-                 
+            # pygame.draw.rect(self.screen,(0,0,0),brect)
             if brect.colliderect(xc):
                 if self.rect_pre.x < brect.x and dx<0: #왼쪽에
                     pass
@@ -334,32 +438,126 @@ class Player():
         #     if self.snd_dic['weapon'] is not None:
         #         self.snd_dic['weapon'].play()
                 
+        if pygame.sprite.spritecollide(self, self.parent.group_hp, True):
+            self.hp += 1
+            if self.snd_dic['coin'] is not None:
+                self.snd_dic['coin'].play()
+                
         if pygame.sprite.spritecollide(self, self.parent.group_coin, True):
             self.score += 10
             if self.snd_dic['coin'] is not None:
                 self.snd_dic['coin'].play()
         #몬스터
         if pygame.sprite.spritecollide(self, self.parent.group_monster, False):
+            self.hp -= 1
             if self.snd_dic['game_over'] is not None:
                 self.snd_dic['game_over'].play()
                 # self.game_reset(True)
-            self.gameover = True
+            if self.hp <= 0:
+                self.gameover = True
+            else:
+                # self.level = self.parent.map_change(self.level)
+                self.image = self.get_img(self.level)
+                self.rect.x,self.rect.y = self.get_position(self.level)      
+                self.rect_pre = self.rect.copy()
 
+        #몬스터 총알
+        if pygame.sprite.spritecollide(self, self.parent.group_bulletMonster, True):
+            self.hp -= 1
+            if self.snd_dic['game_over'] is not None:
+                self.snd_dic['game_over'].play()
+                # self.game_reset(True)
+            if self.hp <= 0:
+                self.gameover = True
+            else:
+                # self.level = self.parent.map_change(self.level)
+                self.image = self.get_img(self.level)
+                self.rect.x,self.rect.y = self.get_position(self.level)      
+                self.rect_pre = self.rect.copy()
+                
         # 용암 충돌확인? 
         if pygame.sprite.spritecollide(self, self.parent.group_lava, False):
+            self.hp -= 1
             if self.snd_dic['game_over'] is not None:
                 self.snd_dic['game_over'].play()
                 # self.game_reset(True)
-            self.gameover = True
-            
-        if pygame.sprite.spritecollide(self, self.parent.group_exitDoor, False):
-            self.level += 1
+            if self.hp <= 0:
+                self.gameover = True
+            else:
+                # self.level = self.parent.map_change(self.level)
+                self.image = self.get_img(self.level)
+                self.rect.x,self.rect.y = self.get_position(self.level)      
+                self.rect_pre = self.rect.copy()
+                
+        exit_door = pygame.sprite.spritecollide(self, self.parent.group_exitDoor, False)
+        if len(exit_door):
+            level = exit_door[0].next_level#맵점프
+            # print(f'Exit Door : {level}:{self.level}')
+            if level != -1:
+                self.level = level
+            else:
+                self.level += 1
+                   
+            if self.level > self.parent.lastExt:
+                self.MissionCompleted = True
+            else:
+                # self.level += 1            
+                self.level = self.parent.map_change(self.level)
+                self.image = self.get_img(self.level)
+                self.rect.x,self.rect.y = self.get_position(self.level)      
+                self.rect_pre = self.rect.copy()
+                self.level_real = self.level
             
+            if self.level in self.jump_set_level:
+                self.JUMP = self.jump_set_level[self.level]
+                
+            if self.level in self.speed_set_level:
+                self.speed = self.speed_set_level[self.level]
+                
+    def check_level_real(self):
+        if self.level_real != self.level:
+            self.level_real = self.level
             self.level = self.parent.map_change(self.level)
             self.image = self.get_img(self.level)
             self.rect.x,self.rect.y = self.get_position(self.level)      
             self.rect_pre = self.rect.copy()
-                
+            self.level_real = self.level
+            
+    def draw_message_success(self):
+        if self.MissionCompleted==False:
+            if pygame.time.get_ticks() > self.gametime_tick:
+                self.gametime_tick = pygame.time.get_ticks() + 10
+                self.gametime += 1
+            return False
+        
+        msg = f'Mission completed!!'
+        img = self.mfont50.render(msg, True, (192,0,0))
+        img.set_alpha(100)
+        rect = img.get_rect()
+        rect.centerx = self.screen.get_width()/2
+        rect.centery = self.screen.get_height()/2-100
+        self.screen.blit(img, rect)
+        
+        msg = f'재시작 (Click)'
+        img = self.mfont40.render(msg, True, (192,0,0))
+        img.set_alpha(100)
+        rect = img.get_rect()
+        rect.centerx = self.screen.get_width()/2
+        rect.centery = self.screen.get_height()/2
+        self.screen.blit(img, rect)
+        
+        if rect.collidepoint(pygame.mouse.get_pos()):
+            pygame.draw.rect(self.screen,(255,0,0),rect,2)
+            if pygame.mouse.get_pressed()[0]:
+                self.game_reset(True)
+                self.level = 1            
+                self.level = self.parent.map_change(self.level)
+                self.image = self.get_img(self.level)
+                self.rect.x,self.rect.y = self.get_position(self.level)      
+                self.rect_pre = self.rect.copy()
+            
+        return True
+        
     def draw_message(self, msg:str, color:tuple, x:int, y:int):
         msg = f'{msg}'
         img = self.mfont20.render(msg, True, color)
@@ -383,7 +581,29 @@ class Player():
         self.msg_weapon_color = color
         self.msg_weapon_text = text
         
+    def set_msg_hp(self, x=10,y=130, color = (0,0,0), text = 'HP : '):
+        self.msg_hp_x = x
+        self.msg_hp_y = y
+        self.msg_hp_color = color
+        self.msg_hp_text = text
+        
+    def set_msg_jumpcnt(self, x=10,y=170, color = (0,0,0), text = '점프 : '):
+        self.msg_jumpcnt_x = x
+        self.msg_jumpcnt_y = y
+        self.msg_jumpcnt_color = color
+        self.msg_jumpcnt_text = text
+        
+    def set_msg_gameTime(self, x=10,y=210, color = (0,0,0), text = '시간 : '):
+        self.msg_gametime_x = x
+        self.msg_gametime_y = y
+        self.msg_gametime_color = color
+        self.msg_gametime_text = text
+        
     def draw(self):
+        if self.is_start:
+            self.is_start = False
+            self.game_reset(True)
+            
         if self.msg_score_text is not None:
             self.draw_message(f'{self.msg_score_text}{self.score}',
                             self.msg_score_color, 
@@ -402,8 +622,30 @@ class Player():
                             x=self.msg_weapon_x,
                             y=self.msg_weapon_y)
             
+        if self.msg_hp_text is not None:
+            self.draw_message(f'{self.msg_hp_text}{self.hp}',
+                            self.msg_hp_color, 
+                            x=self.msg_hp_x,
+                            y=self.msg_hp_y)
+            
+        if self.msg_jumpcnt_text is not None:
+            self.draw_message(f'{self.msg_jumpcnt_text}{self.jump_cnt}',
+                            self.msg_jumpcnt_color, 
+                            x=self.msg_jumpcnt_x,
+                            y=self.msg_jumpcnt_y)
+            
+        if self.msg_gametime_text is not None:
+            self.draw_message(f'{self.msg_gametime_text}{self.gametime/100:.2f}초',
+                            self.msg_gametime_color, 
+                            x=self.msg_gametime_x,
+                            y=self.msg_gametime_y)
+            
         if self.game_over_process():
-            self.key_pressed()        
+            self.check_level_real()
+            if self.draw_message_success():
+                pass
+            else:
+                self.key_pressed()        
             self.check_img_dir()
             self.rect.y += self.jump_process()
             self.check_img_screen_limit()        
